#!/usr/bin/perl5

# ndc.cron.pl
# cron job to determine if DNS has been updated and, if so, force a
# dump of the databases to disk.
# Uses ndc to do the dirty work
#
# Copyright (C) 1999 Stephen Carville
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

# edit these for your own system.
my $DDNSHOME="/root/DHCP-DNS";
my $NDC="/usr/sbin/ndc restart";

my $DHCPD_TEMP="$DDNSHOME/dhcpd.leases.last";
my $DNS_TEMP="$DDNSHOME/dns.last";
my $LOCKFILE="$DDNSHOME/lock";

# sleep for this long between tries
my $SNOOZE=10;
# only back off five times if locked out 
my $MAXTRIES=5;

my($ref,$data,$snooze,$command);
my ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,$atime,$mtime,$ctime,$blksize,$blocks);

# get timestamp on leases backup file
($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,$atime,$mtime,$ctime,
 $blksize,$blocks)=stat($DHCPD_TEMP);
$data=$mtime;

# get timestamp on dns ref file
($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,$atime,$mtime,$ctime,
 $blksize,$blocks)=stat($DNS_TEMP);
$ref=$mtime;

if($ref >= $data) {
  exit 0;
}

# make sure no DNS updates are in process
$snooze=0;
while (-e "$LOCKFILE") {
  sleep $SNOOZE;
  $snooze++;
  if ($snooze > $MAXTRIES) {
    print STDERR "dns.cron.pl -- too many timeouts!\n";
    exit 0;
  }
}

# if we get this far then run the update command
$command="$NDC\n";
`$command`;

# touch the time stamp file
`touch $DNS_TEMP`;
